extension YModel on Map {
  T? byModel<T>(String key, T? Function(Map<dynamic, dynamic> value) onModel) {
    if (this[key] == null) return null;
    return onModel.call(this[key]);
  }

  List<T>? byList<T>(String key, T Function(Map<dynamic, dynamic> value) onModel) {
    if (this[key] == null) return null;
    if (this[key] is! List) return null;
    return (this[key] as List).map((e) => onModel.call(e)).toList();
  }

  String? byString(String key) {
    if (this[key] == null) return null;
    return this[key]?.toString();
  }

  int? byInt(String key) {
    if (this[key] == null) return null;
    return double.parse(byString(key)!).toInt();
  }

  bool? byBool(String key) {
    if (this[key] == null) return null;
    return byString(key) == "true";
  }

  T? byEnum<T, E>(String key, T Function(String? value) onModel) {
    if (this[key] == null) return null;
    return onModel.call(this[key]?.toString());
  }
}

// class _DemoModel {
//   _DemoSubModel? modelNull;
//   _DemoSubModel modelNotNull = _DemoSubModel();
//   List<_DemoSubModel>? listNull;
//   List<_DemoSubModel> listNotNull = [];
//   String? stringNull;
//   String stringNotNull = "";
//   int? intNull;
//   int intNotNull = 0;
//   bool? boolNull;
//   bool boolNotNull = false;
//   _DemoEnum? enumNull;
//   _DemoEnum enumNotNull = _DemoEnum.UnKnown;
//
//   _DemoModel.fromMap(Map<dynamic, dynamic> map) {
//     modelNull = map.byModel("modelNull", (value) => _DemoSubModel.fromMap(value));
//     modelNotNull = map.byModel("modelNotNull", (value) => _DemoSubModel.fromMap(value)) ?? _DemoSubModel();
//     listNull = map.byList("listNull", (value) => _DemoSubModel.fromMap(value));
//     listNotNull = map.byList("listNotNull", (value) => _DemoSubModel.fromMap(value)) ?? [];
//     stringNull = map.byString("stringNull");
//     stringNotNull = map.byString("stringNotNull") ?? "";
//     intNull = map.byInt("intNull");
//     intNotNull = map.byInt("intNotNull") ?? 0;
//     boolNull = map.byBool("boolNull");
//     boolNotNull = map.byBool("boolNotNull") ?? false;
//     enumNull = map.byEnum("enumNull", (value) => _DemoEnum.from(value));
//     enumNotNull = map.byEnum("enumNotNull", (value) => _DemoEnum.from(value)) ?? _DemoEnum.UnKnown;
//   }
// }
//
// enum _DemoEnum {
//   UnKnown,
//   Enum1(rawValue: "Enum1"),
//   ;
//
//   final String? rawValue;
//
//   const _DemoEnum({this.rawValue});
//
//   static _DemoEnum from(String? rawValue) => values.firstWhere((x) => x.rawValue == rawValue, orElse: () => UnKnown);
// }
//
// class _DemoSubModel {
//   _DemoSubModel();
//
//   _DemoSubModel.fromMap(Map<dynamic, dynamic> map);
// }
